/****************************************************************************
*                                                                           *
*     ______     ______     __   __     ______     __  __     ______        *
*    /\___  \   /\___  \   /\ "-.\ \   /\  __ \   /\ \/ /    /\  ___\       *
*    \/_/  /__  \/_/  /__  \ \ \-.  \  \ \  __ \  \ \  _"-.  \ \  __\       *
*      /\_____\   /\_____\  \ \_\\"\_\  \ \_\ \_\  \ \_\ \_\  \ \_____\     *
*      \/_____/   \/_____/   \/_/ \/_/   \/_/\/_/   \/_/\/_/   \/_____/     *
*                                                                           *
*                                                                           *
*                                                                           *
*   This file is part of zZNAKE                                             *
*                                                                           *
*   Copyright (C) 2012  Fabien LOISON <http://www.flogisoft.com>            *
*                                                                           *
*   zZNAKE is free software: you can redistribute it and/or modify          *
*   it under the terms of the GNU General Public License as published by    *
*   the Free Software Foundation, either version 3 of the License, or       *
*   (at your option) any later version.                                     *
*                                                                           *
*   This program is distributed in the hope that it will be useful,         *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*   GNU General Public License for more details.                            *
*                                                                           *
*   You should have received a copy of the GNU General Public License       *
*   along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
*                                                                           *
****************************************************************************/


#include "_header.h"


void game_new()
{
	int i;
	UBYTE j;

	//Initialize the new game
	Game game;
	game.level = 0;
	game.speed = 9;

	//Test
	game_load_level(&game, 0);

	i = 0;
	while (1)
	{
		j = joypad();
		if (j & J_UP && game.direction != DIRECTION_DOWN)
			game.direction = DIRECTION_UP;
		else if (j & J_RIGHT && game.direction != DIRECTION_LEFT)
			game.direction = DIRECTION_RIGHT;
		else if (j & J_DOWN && game.direction != DIRECTION_UP)
			game.direction = DIRECTION_DOWN;
		else if (j & J_LEFT && game.direction != DIRECTION_RIGHT)
			game.direction = DIRECTION_LEFT;

		if (i == 0)
		{
			if (map_check_snake_collide(&game))
			{
				return;
			}
			game_move_snake(&game);
			wait_vbl_done();
			game_refresh(&game);
			i = game.speed;
		}
		else
		{
			i--;
			wait_vbl_done();
		}
	}
}


void game_load_level(Game *game, int level)
{
	int i;

	//Set the right level number
	game->level = level;

	//Copy the game board
	memcpy(game->map, levels + level * LEVEL_SIZE, LEVEL_SIZE);

	//Initialize the snake
	for (i=3 ; i<SNK_MAX_LENGTH ; i++)
	{
		game->snake[i].x = 0;
		game->snake[i].y = 0;
		game->snake[i].alive = 0;
	}
	game->snake[0].x = 2; game->snake[0].y = 2; game->snake[0].alive = 1;
	game->snake[1].x = 3; game->snake[1].y = 2; game->snake[1].alive = 1;
	game->snake[2].x = 4; game->snake[2].y = 2; game->snake[2].alive = 1;

	//Initialize the direction
	game->direction = DIRECTION_RIGHT;
}


void game_refresh(Game *game) {
	int i;

	//Set the background
	memcpy(game->map, levels + game->level * LEVEL_SIZE, LEVEL_SIZE);

	//Put the snake on the board
	for (i=0 ; i<SNK_MAX_LENGTH ; i++)
	{
		if (game->snake[i].alive)
		{
			map_set_xy(game, game->snake[i].x, game->snake[i].y, TILE_SNK_BODY);
		}
	}
	set_bkg_tiles(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, game->map);
}


void game_move_snake(Game *game)
{
	int i;

	for (i=0 ; i<SNK_MAX_LENGTH-1 ; i++)
	{
		if (game->snake[i+1].alive)
		{
			game->snake[i].x = game->snake[i+1].x;
			game->snake[i].y = game->snake[i+1].y;
		}
		else
		{
			switch (game->direction)
			{
				case DIRECTION_UP:
					game->snake[i].y--;
					break;
				case DIRECTION_RIGHT:
					game->snake[i].x++;
					break;
				case DIRECTION_DOWN:
					game->snake[i].y++;
					break;
				case DIRECTION_LEFT:
					game->snake[i].x--;
					break;
			}
			//horizontal check
			if (game->snake[i].x >= SCREEN_WIDTH)
				game->snake[i].x = 0;
			else if (game->snake[i].x < 0)
				game->snake[i].x = SCREEN_WIDTH - 1;
			//Vertical check
			if (game->snake[i].y >= SCREEN_HEIGHT)
				game->snake[i].y = 0;
			else if (game->snake[i].y < 0)
				game->snake[i].y = SCREEN_HEIGHT - 1;
			break;
		}
	}
}


void map_set_xy(Game* game, int x, int y, unsigned char tile)
{
	game->map[y * SCREEN_WIDTH + x] = tile;
}


unsigned char map_get_xy(Game* game, int x, int y)
{
	return game->map[y * SCREEN_WIDTH + x];
}


int map_check_snake_collide(Game *game)
{
	int i;

	for (i=0 ; i<SNK_MAX_LENGTH-1 ; i++)
	{
		if (!game->snake[i+1].alive)
		{
			int x = game->snake[i].x;
			int y = game->snake[i].y;
			unsigned char t = 0;
			switch (game->direction)
			{
				case DIRECTION_UP:
					y--;
					break;
				case DIRECTION_RIGHT:
					x++;
					break;
				case DIRECTION_DOWN:
					y++;
					break;
				case DIRECTION_LEFT:
					x--;
					break;
			}
			t = map_get_xy(game, x, y);
			//horizontal check
			if (t != TILE_VOID)
				return 1;
			else
				return 0;
		}
	}
}
